/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */
 
#if CONFIG_BOARD_CUSTOM_NRF52832 || CONFIG_BOARD_CUSTOM_NRF5340_CPUAPP 


#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/spi.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <zephyr/sys/printk.h>
#include "data.h"
#include "error.h"

#define LIS2DH12_REG_READ_MSK    0xC0

// register address
#define LIS2DH12_REG_ADDR_STATUS_REG_AUX 0x07
#define LIS2DH12_REG_ADDR_OUT_TEMP_L     0x0C
#define LIS2DH12_REG_ADDR_OUT_TEMP_H     0x0D
#define LIS2DH12_REG_ADDR_WHO_AM_I       0x0F
#define LIS2DH12_REG_ADDR_CTRL_REG0      0x1E
#define LIS2DH12_REG_ADDR_TEMP_CFG_REG   0x1F
#define LIS2DH12_REG_ADDR_CTRL_REG1      0x20
#define LIS2DH12_REG_ADDR_CTRL_REG2      0x21
#define LIS2DH12_REG_ADDR_CTRL_REG3      0x22
#define LIS2DH12_REG_ADDR_CTRL_REG4      0x23
#define LIS2DH12_REG_ADDR_CTRL_REG5      0x24
#define LIS2DH12_REG_ADDR_CTRL_REG6      0x25
#define LIS2DH12_REG_ADDR_REFERENCE      0x26
#define LIS2DH12_REG_ADDR_STATUS_REG     0x27
#define LIS2DH12_REG_ADDR_OUT_X_L        0x28
#define LIS2DH12_REG_ADDR_OUT_X_H        0x29
#define LIS2DH12_REG_ADDR_OUT_Y_L        0x2A
#define LIS2DH12_REG_ADDR_OUT_Y_H        0x2B
#define LIS2DH12_REG_ADDR_OUT_Z_L        0x2C
#define LIS2DH12_REG_ADDR_OUT_Z_H        0x2D
#define LIS2DH12_REG_ADDR_FIFO_CTRL_REG  0x2E
#define LIS2DH12_REG_ADDR_FIFO_SRC_REG   0x2F
#define LIS2DH12_REG_ADDR_INT1_CFG       0x30
#define LIS2DH12_REG_ADDR_INT1_SRC       0x31
#define LIS2DH12_REG_ADDR_INT1_THS       0x32
#define LIS2DH12_REG_ADDR_INT1_DURATION  0x33
#define LIS2DH12_REG_ADDR_INT2_CFG       0x34
#define LIS2DH12_REG_ADDR_INT2_SRC       0x35
#define LIS2DH12_REG_ADDR_INT2_THS       0x36
#define LIS2DH12_REG_ADDR_INT2_DURATION  0x37
#define LIS2DH12_REG_ADDR_CLICK_CFG      0x38
#define LIS2DH12_REG_ADDR_CLICK_SRC      0x39
#define LIS2DH12_REG_ADDR_CLICK_THS      0x3A
#define LIS2DH12_REG_ADDR_TIME_LIMIT     0x3B
#define LIS2DH12_REG_ADDR_TIME_LATENCY   0x3C
#define LIS2DH12_REG_ADDR_TIME_WINDOW    0x3D
#define LIS2DH12_REG_ADDR_ACT_THS        0x3E
#define LIS2DH12_REG_ADDR_ACT_DUR        0x3F

// device ID
#define LIS2DH12_DEVICE_ID 0x33
#define GSSPI_NODE DT_NODELABEL(gsspi)

// SPI master functionality
const struct device* _gSpiDev = DEVICE_DT_GET(GSSPI_NODE);
//static struct k_poll_signal spi_done_sig = K_POLL_SIGNAL_INITIALIZER(spi_done_sig);

struct spi_cs_control spim_cs = {
    .gpio = SPI_CS_GPIOS_DT_SPEC_GET(DT_NODELABEL(reg_gsspi)),
    .delay = 0,
};

static const struct spi_config _gSpiCfg = {
    .operation = SPI_WORD_SET(8)  |
                 SPI_TRANSFER_MSB |
                 SPI_MODE_CPOL    |
                 SPI_MODE_CPHA,
    .frequency = 8000000,
    .slave = 0,
    .cs = &spim_cs,
};

LOG_MODULE_REGISTER(gsensor);

static int _lis2dh12RegRead(const uint8_t regAddr, uint8_t* regVal) {
    int ret;
    uint8_t txBuf[1] = { regAddr | LIS2DH12_REG_READ_MSK };

    const struct spi_buf txSpiBuf = {
        .buf = txBuf,
        .len = sizeof(txBuf)
    };

    const struct spi_buf_set txSpiBufSet = {
        .buffers = &txSpiBuf,
        .count   = 1
    };

    uint8_t rxBuf[2];
    struct spi_buf rxSpiBuf = {
        .buf = rxBuf,
        .len = sizeof(rxBuf)
    };
    const struct spi_buf_set rxSpiBufSet = { .buffers = &rxSpiBuf, .count = 1 };
    ret = spi_transceive(_gSpiDev, &_gSpiCfg, &txSpiBufSet, &rxSpiBufSet);
    if (ret < 0) {
        printk("Reading LIS2DH12 register failed, regAddr=0x%x, err=%d\n", regAddr, ret);
        return ERROR_GSENSOR_READ_FAIL;
    }

    *regVal = rxBuf[1];

    return 0;
}

static int _lis2dh12RegWhoAmIRead(uint8_t* id) {
    if (_lis2dh12RegRead(LIS2DH12_REG_ADDR_WHO_AM_I, id) < 0) {
        LOG_ERR("Reading LIS2DH12 WHO_AM_I register failed\n");
        return ERROR_GSENSOR_READ_FAIL;
    }

    return 0;
}

bool _lis2dh12RegWhoAmICheck(void) {
    uint8_t id;
    if (_lis2dh12RegWhoAmIRead(&id) < 0) {
        return false;
    }

    if (id != LIS2DH12_DEVICE_ID) {
        LOG_ERR("LIS2DH12 ID check failed, id=0x%x\n", id);
        return false;
    }

    printk("LIS2DH12 ID check ok, id=0x%x\n", id);
    return true;
}

int lis2dh12Init(void) {
    if (!device_is_ready(_gSpiDev)) {
        printk("lis2dh12 device isn't ready!\n");
        return ERROR_GSENSOR_INITIALIZE_FAIL;
    }

    if(!device_is_ready(spim_cs.gpio.port)){
        printk("lis2dh12 chip select device isn't ready!\n");
        return ERROR_GSENSOR_INITIALIZE_FAIL;
    }

    printk("lis2dh12 device is ready!\n");
    if (!_lis2dh12RegWhoAmICheck()) {
        return ERROR_GSENSOR_READ_FAIL;
    }
    printk("lis2dh12 ID check pass!\n");

    return 0;
}

#endif